#!/usr/bin/env stack
-- stack --resolver lts-11.5 script

{-# LANGUAGE OverloadedStrings #-}

import Prelude (($))
import Control.Monad (return, mapM_)
import Data.Char (toUpper)
import Data.Functor (fmap)
import Data.Maybe (Maybe(..), catMaybes)
import qualified Data.Map as M
import Data.Text (pack)
import Data.Text.IO (getContents, appendFile, writeFile)
import Text.Pandoc.Definition (Pandoc(Pandoc), Block(CodeBlock))
import Text.Pandoc.JSON (toJSONFilter)
import System.IO (IO)
import System.FilePath

main :: IO ()
main = toJSONFilter doItAll

doItAll :: Pandoc -> IO Pandoc
doItAll (Pandoc meta blocks) = do
  mapM_ (\f -> writeFile f "") (listOutputFiles blocks)
  mapM_ catCodeBlocks blocks
  return (Pandoc meta blocks)

attrFile :: Block -> Maybe FilePath
attrFile (CodeBlock (_,_,attr) _) = M.lookup "file" (M.fromList attr)
attrFile _ = Nothing

listOutputFiles :: [Block] -> [FilePath]
listOutputFiles blocks = catMaybes (fmap attrFile blocks)

catCodeBlocks :: Block -> IO ()
catCodeBlocks (CodeBlock (id,classes,attr) s) =
  case attrFile (CodeBlock (id,classes,attr) s) of
    Just file -> do
      appendFile file (pack s)
      appendFile file "\n"
    Nothing -> return ()
catCodeBlocks _ = return ()
