#!/usr/bin/env stack
-- stack --resolver lts-11.11 script

{-# LANGUAGE OverloadedStrings #-}

import Prelude (($))
import Control.Monad (return, mapM_)
import Data.Char (toUpper)
import Data.Default (def)
import Data.Functor (fmap)
import Data.List ((++), nub)
import Data.Maybe (Maybe(..), catMaybes)
import qualified Data.Map as M
import Data.Text (pack)
import Data.Text.IO (getContents, appendFile, writeFile, readFile)
import Text.Pandoc.Class (runIOorExplode)
import Text.Pandoc.Definition (Pandoc(Pandoc), Block(CodeBlock))
import Text.Pandoc.Extensions (pandocExtensions)
import Text.Pandoc.JSON (toJSONFilter)
import Text.Pandoc.Options (ReaderOptions(..))
import Text.Pandoc.Readers (readMarkdown)
import Text.Show (show)
import System.IO (IO, putStrLn)
import System.FilePath (FilePath)

import Development.Shake
import Development.Shake.Command
import Development.Shake.FilePath
import Development.Shake.Util

import Debug.Trace (trace)

main :: IO ()
main = shakeArgs shakeOptions $ do
    (Pandoc meta blocks) <- liftIO $ readPandoc "README.md"
    let oFiles = listOutputFiles blocks

    want (["README.html"] ++ oFiles)

    phony "clean" $ do
        removeFilesAfter "." ["README.html"]
        removeFilesAfter "." oFiles

    "README.html" %> \out -> do
        need ["README.md", "css/buttondown.css", "css/style.css"]
        command_ [] "pandoc" ["README.md", "-s", "-o", out, "--css=css/buttondown.css", "--css=css/style.css"]

    oFiles &%> \out -> do
        need ["README.md"]
        liftIO (catItAll "README.md")


readPandoc :: FilePath -> IO Pandoc
readPandoc filepath = do
  contents <- readFile filepath
  runIOorExplode $ readMarkdown (def {readerExtensions = pandocExtensions}) contents

catItAll :: FilePath -> IO ()
catItAll filepath = do
  (Pandoc meta blocks) <- readPandoc filepath
  mapM_ (\f -> writeFile f "") (listOutputFiles blocks)
  mapM_ catCodeBlocks blocks

attrFile :: Block -> Maybe FilePath
attrFile (CodeBlock (id,cl,attr) s) = M.lookup "file" (M.fromList attr)
attrFile _ = Nothing

listOutputFiles :: [Block] -> [FilePath]
listOutputFiles blocks = nub $ catMaybes (fmap attrFile blocks)

catCodeBlocks :: Block -> IO ()
catCodeBlocks (CodeBlock (id,classes,attr) s) =
  case attrFile (CodeBlock (id,classes,attr) s) of
    Just file -> do
      appendFile file (pack s)
      appendFile file "\n\n"
    Nothing -> return ()
catCodeBlocks _ = return ()
